/*
 * basic.cpp
 *
 *  Created on: 27/03/2013
 *      Author: victor
 */
#include <ros/ros.h>
#include <sensor_msgs/LaserScan.h>
#include <nav_msgs/Odometry.h>
#include <geometry_msgs/Twist.h>
#include <tf/transform_broadcaster.h>
#include <tf/transform_listener.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
using namespace std;

bool newLaserData=false;
vector<float> ranges;

//recibe lecturas del laser y las imprime
void laserCallback(const sensor_msgs::LaserScan::ConstPtr& msg){
	static int rangescount = (msg->angle_max - msg->angle_min) / msg->angle_increment;
	//imprimimos info del laser
	float minAng = msg->angle_min;
	float maxAng = msg->angle_max;
	float angularRes = msg->angle_increment;
	printf("Laser data: From:%f to:%f by:%f Total:%d\n", minAng * 180.0 / M_PI, maxAng * 180.0 / M_PI, angularRes * 180.0 / M_PI, rangescount);
	ranges.resize(rangescount);
	//imprimimos valores del scan
	for (int i = 0; i<rangescount; i=i+10) { //0 to rangescount
		printf("scan at %f degrees = %f\n", (msg->angle_min + i*msg->angle_increment)*180.0/M_PI, msg->ranges[i]);
		ranges.push_back(msg->ranges[i]);
		}
	newLaserData = true;
	}

//recibe pose y velocidad del robot y la imprime
void odometryCallback(const nav_msgs::Odometry::ConstPtr& msg) {
	printf("Robot odometry at (%f , %f) yaw=%f\n",msg->pose.pose.position.x,msg->pose.pose.position.y,tf::getYaw(msg->pose.pose.orientation));
	printf("Robot current vx=%f vang=%f\n",msg->twist.twist.linear.x,msg->twist.twist.angular.z);
	}


int main(int argc,char **argv){
	ros::init(argc, argv, "ejemplo_basico");
	ros::Time::init();
	ros::NodeHandle n;
	ros::Subscriber lasersub = n.subscribe("/scan", 1, &laserCallback);
	ros::Subscriber odomsub = n.subscribe("/RosAria/pose", 1, &odometryCallback);
	ros::Publisher cmdPub = n.advertise<geometry_msgs::Twist>("cmd_vel", 1);
	ros::Rate loop_rate(10);
	for(;;){
		ros::spinOnce();
		if(newLaserData){
			//procesar: datos en el vector ranges

			//...

			//enviar velocidad
			geometry_msgs::Twist msg;
			msg.linear.x = 0.0;
			msg.angular.z = 0.2;
			cmdPub.publish(msg);
			loop_rate.sleep();
			}
		}

	}
